@@ -1,27 +1,33 @@
-import { extname } from 'path';
+import { dirname, extname } from 'path';
 
 import { createFilter } from '@rollup/pluginutils';
 import getCommonDir from 'commondir';
 
 import { peerDependencies } from '../package.json';
 
+import analyzeTopLevelStatements from './analyze-top-level-statements';
+
 import {
   getDynamicPackagesEntryIntro,
   getDynamicPackagesModule,
-  isModuleRegistrationProxy
+  isDynamicModuleImport
 } from './dynamic-packages-manager';
 import getDynamicRequirePaths from './dynamic-require-paths';
 import {
   DYNAMIC_JSON_PREFIX,
   DYNAMIC_PACKAGES_ID,
+  DYNAMIC_REGISTER_SUFFIX,
+  EXPORTS_SUFFIX,
   EXTERNAL_SUFFIX,
   getHelpersModule,
   HELPERS_ID,
   isWrappedId,
+  MODULE_SUFFIX,
   PROXY_SUFFIX,
   unwrapId
 } from './helpers';
-import { setIsCjsPromise } from './is-cjs';
+import { setCommonJSMetaPromise } from './is-cjs';
+import { hasCjsKeywords } from './parse';
 import {
   getDynamicJsonProxy,
   getDynamicRequireProxy,
@@ -30,18 +36,16 @@ import {
   getUnknownRequireProxy
 } from './proxies';
 import getResolveId from './resolve-id';
-import {
-  checkEsModule,
-  hasCjsKeywords,
-  normalizePathSlashes,
-  transformCommonjs
-} from './transform';
+import validateRollupVersion from './rollup-version';
+import transformCommonjs from './transform-commonjs';
+import { getName, getVirtualPathForDynamicRequirePath, normalizePathSlashes } from './utils';
 
 export default function commonjs(options = {}) {
   const extensions = options.extensions || ['.js'];
   const filter = createFilter(options.include, options.exclude);
   const {
     ignoreGlobal,
+    ignoreDynamicRequires,
     requireReturnsDefault: requireReturnsDefaultOption,
     esmExternals
   } = options;
@@ -56,6 +60,8 @@ export default function commonjs(options = {}) {
       : Array.isArray(esmExternals)
       ? ((esmExternalIds = new Set(esmExternals)), (id) => esmExternalIds.has(id))
       : () => esmExternals;
+  const defaultIsModuleExports =
+    typeof options.defaultIsModuleExports === 'boolean' ? options.defaultIsModuleExports : 'auto';
 
   const { dynamicRequireModuleSet, dynamicRequireModuleDirPaths } = getDynamicRequirePaths(
     options.dynamicRequireTargets
@@ -67,6 +73,7 @@ export default function commonjs(options = {}) {
 
   const esModulesWithDefaultExport = new Set();
   const esModulesWithNamedExports = new Set();
+  const commonJsMetaPromises = new Map();
 
   const ignoreRequire =
     typeof options.ignore === 'function'
@@ -75,17 +82,32 @@ export default function commonjs(options = {}) {
       ? (id) => options.ignore.includes(id)
       : () => false;
 
+  const getIgnoreTryCatchRequireStatementMode = (id) => {
+    const mode =
+      typeof options.ignoreTryCatch === 'function'
+        ? options.ignoreTryCatch(id)
+        : Array.isArray(options.ignoreTryCatch)
+        ? options.ignoreTryCatch.includes(id)
+        : options.ignoreTryCatch || false;
+
+    return {
+      canConvertRequire: mode !== 'remove' && mode !== true,
+      shouldRemoveRequireStatement: mode === 'remove'
+    };
+  };
+
   const resolveId = getResolveId(extensions);
 
   const sourceMap = options.sourceMap !== false;
 
   function transformAndCheckExports(code, id) {
     if (isDynamicRequireModulesEnabled && this.getModuleInfo(id).isEntry) {
+      // eslint-disable-next-line no-param-reassign
       code =
         getDynamicPackagesEntryIntro(dynamicRequireModuleDirPaths, dynamicRequireModuleSet) + code;
     }
 
-    const { isEsModule, hasDefaultExport, hasNamedExports, ast } = checkEsModule(
+    const { isEsModule, hasDefaultExport, hasNamedExports, ast } = analyzeTopLevelStatements(
       this.parse,
       code,
       id
@@ -104,8 +126,12 @@ export default function commonjs(options = {}) {
       return { meta: { commonjs: { isCommonJS: false } } };
     }
 
-    // avoid wrapping in createCommonjsModule, as this is a commonjsRegister call
-    const disableWrap = isModuleRegistrationProxy(id, dynamicRequireModuleSet);
+    // avoid wrapping as this is a commonjsRegister call
+    const disableWrap = isWrappedId(id, DYNAMIC_REGISTER_SUFFIX);
+    if (disableWrap) {
+      // eslint-disable-next-line no-param-reassign
+      id = unwrapId(id, DYNAMIC_REGISTER_SUFFIX);
+    }
 
     return transformCommonjs(
       this.parse,
@@ -114,12 +140,15 @@ export default function commonjs(options = {}) {
       isEsModule,
       ignoreGlobal || isEsModule,
       ignoreRequire,
+      ignoreDynamicRequires && !isDynamicRequireModulesEnabled,
+      getIgnoreTryCatchRequireStatementMode,
       sourceMap,
       isDynamicRequireModulesEnabled,
       dynamicRequireModuleSet,
       disableWrap,
       commonDir,
-      ast
+      ast,
+      defaultIsModuleExports
     );
   }
 
@@ -127,33 +156,58 @@ export default function commonjs(options = {}) {
     name: 'commonjs',
 
     buildStart() {
+      validateRollupVersion(this.meta.rollupVersion, peerDependencies.rollup);
       if (options.namedExports != null) {
         this.warn(
           'The namedExports option from "@rollup/plugin-commonjs" is deprecated. Named exports are now handled automatically.'
         );
       }
-
-      const [major, minor] = this.meta.rollupVersion.split('.').map(Number);
-      const minVersion = peerDependencies.rollup.slice(2);
-      const [minMajor, minMinor] = minVersion.split('.').map(Number);
-      if (major < minMajor || (major === minMajor && minor < minMinor)) {
-        this.error(
-          `Insufficient Rollup version: "@rollup/plugin-commonjs" requires at least rollup@${minVersion} but found rollup@${this.meta.rollupVersion}.`
-        );
-      }
     },
 
     resolveId,
 
     load(id) {
       if (id === HELPERS_ID) {
-        return getHelpersModule(isDynamicRequireModulesEnabled);
+        return getHelpersModule(isDynamicRequireModulesEnabled, ignoreDynamicRequires);
       }
 
       if (id.startsWith(HELPERS_ID)) {
         return getSpecificHelperProxy(id);
       }
 
+      if (isWrappedId(id, MODULE_SUFFIX)) {
+        const actualId = unwrapId(id, MODULE_SUFFIX);
+        let name = getName(actualId);
+        let code;
+        if (isDynamicRequireModulesEnabled) {
+          if (['modulePath', 'commonjsRequire', 'createModule'].includes(name)) {
+            name = `${name}_`;
+          }
+          code =
+            `import {commonjsRequire, createModule} from "${HELPERS_ID}";\n` +
+            `var ${name} = createModule(${JSON.stringify(
+              getVirtualPathForDynamicRequirePath(dirname(actualId), commonDir)
+            )});\n` +
+            `export {${name} as __module}`;
+        } else {
+          code = `var ${name} = {exports: {}}; export {${name} as __module}`;
+        }
+        return {
+          code,
+          syntheticNamedExports: '__module',
+          meta: { commonjs: { isCommonJS: false } }
+        };
+      }
+
+      if (isWrappedId(id, EXPORTS_SUFFIX)) {
+        const actualId = unwrapId(id, EXPORTS_SUFFIX);
+        const name = getName(actualId);
+        return {
+          code: `var ${name} = {}; export {${name} as __exports}`,
+          meta: { commonjs: { isCommonJS: false } }
+        };
+      }
+
       if (isWrappedId(id, EXTERNAL_SUFFIX)) {
         const actualId = unwrapId(id, EXTERNAL_SUFFIX);
         return getUnknownRequireProxy(
@@ -170,8 +224,15 @@ export default function commonjs(options = {}) {
         return getDynamicJsonProxy(id, commonDir);
       }
 
-      if (isModuleRegistrationProxy(id, dynamicRequireModuleSet)) {
-        return getDynamicRequireProxy(normalizePathSlashes(id), commonDir);
+      if (isDynamicModuleImport(id, dynamicRequireModuleSet)) {
+        return `export default require(${JSON.stringify(normalizePathSlashes(id))});`;
+      }
+
+      if (isWrappedId(id, DYNAMIC_REGISTER_SUFFIX)) {
+        return getDynamicRequireProxy(
+          normalizePathSlashes(unwrapId(id, DYNAMIC_REGISTER_SUFFIX)),
+          commonDir
+        );
       }
 
       if (isWrappedId(id, PROXY_SUFFIX)) {
@@ -180,14 +241,21 @@ export default function commonjs(options = {}) {
           actualId,
           getRequireReturnsDefault(actualId),
           esModulesWithDefaultExport,
-          esModulesWithNamedExports
+          esModulesWithNamedExports,
+          commonJsMetaPromises
         );
       }
 
       return null;
     },
 
-    transform(code, id) {
+    transform(code, rawId) {
+      let id = rawId;
+
+      if (isWrappedId(id, DYNAMIC_REGISTER_SUFFIX)) {
+        id = unwrapId(id, DYNAMIC_REGISTER_SUFFIX);
+      }
+
       const extName = extname(id);
       if (
         extName !== '.cjs' &&
@@ -199,21 +267,18 @@ export default function commonjs(options = {}) {
       }
 
       try {
-        return transformAndCheckExports.call(this, code, id);
+        return transformAndCheckExports.call(this, code, rawId);
       } catch (err) {
         return this.error(err, err.loc);
       }
     },
 
-    moduleParsed({ id, meta: { commonjs } }) {
-      if (commonjs) {
-        const isCjs = commonjs.isCommonJS;
-        if (isCjs != null) {
-          setIsCjsPromise(id, isCjs);
-          return;
-        }
+    moduleParsed({ id, meta: { commonjs: commonjsMeta } }) {
+      if (commonjsMeta && commonjsMeta.isCommonJS != null) {
+        setCommonJSMetaPromise(commonJsMetaPromises, id, commonjsMeta);
+        return;
       }
-      setIsCjsPromise(id, null);
+      setCommonJSMetaPromise(commonJsMetaPromises, id, null);
     }
   };
 }
