@@ -37,10 +37,7 @@ export function wasm(options: RollupWasmOptions = {}): Plugin {
       return Promise.all([fs.promises.stat(id), fs.promises.readFile(id)]).then(
         ([stats, buffer]) => {
           if ((maxFileSize && stats.size > maxFileSize) || maxFileSize === 0) {
-            const hash = createHash('sha1')
-              .update(buffer)
-              .digest('hex')
-              .substr(0, 16);
+            const hash = createHash('sha1').update(buffer).digest('hex').substr(0, 16);
 
             const filename = `${hash}.wasm`;
             const publicFilepath = `${publicPath}${filename}`;
@@ -76,8 +73,13 @@ export function wasm(options: RollupWasmOptions = {}): Plugin {
           src = null;
         }
 
-        return `import { _loadWasmModule } from ${JSON.stringify(HELPERS_ID)};
-export default function(imports){return _loadWasmModule(${+isSync}, ${publicFilepath}, ${src}, imports)}`;
+        return {
+          map: {
+            mappings: ''
+          },
+          code: `import { _loadWasmModule } from ${JSON.stringify(HELPERS_ID)};
+export default function(imports){return _loadWasmModule(${+isSync}, ${publicFilepath}, ${src}, imports)}`
+        };
       }
       return null;
     },
